@@ -1,4 +1,4 @@
-/* $Id: hparser.c,v 2.92 2003/08/17 18:19:13 gisle Exp $
+/* $Id: hparser.c,v 2.95 2004/04/01 11:56:37 gisle Exp $
  *
  * Copyright 1999-2002, Gisle Aas
  * Copyright 1999-2000, Michael A. Chase
@@ -414,7 +414,10 @@ report_event(PSTATE* p_state,
 			sv_lower(aTHX_ attrname);
 
 		    if (argcode == ARG_ATTR) {
-			if (!hv_store_ent(hv, attrname, attrval, 0)) {
+			U32 hash;
+			PERL_HASH(hash, SvPVX(attrname), SvCUR(attrname));
+			if (hv_exists_ent(hv, attrname, hash) ||
+			    !hv_store_ent(hv, attrname, attrval, hash)) {
 			    SvREFCNT_dec(attrval);
 			}
 			SvREFCNT_dec(attrname);
@@ -1606,6 +1609,8 @@ parse(pTHX_
 		    token.beg = s + 1;
 		    token.end = end;
 		    report_event(p_state, E_COMMENT, s, end, &token, 1, self);
+		    SvREFCNT_dec(p_state->buf);
+		    p_state->buf = 0;
 		}
 		else {
 		    goto REST_IS_TEXT;
